/* This file can be redistributed and used under GNU Library General Public
 * License. (c) 2000 Micha Moskal <malekith@topnet.pl>. 
 * $Id: menu.cc,v 1.10 2000/11/12 17:21:20 malekith Exp $ */

#define _V_PRIVATE

#include <v/v.h>
#include <v/term.h>

#include <ctype.h>

bool MenuItem::run()
{
	if (children) {
		active = children;
		parent->draw();
		return false;
	}
	
	if (cmd) {
		// so that menu is closed, before command is run ... :
		send_command(ev_menu_stop);
		// and send the command
		send_command(cmd);
	}
	
	return true;		
}

void MenuItem::handle_event(Event &e)
{
	Widget::handle_event(e);
	
	if (active && e.type == Event::command && e.data == ev_menu_stop) {
		active = 0;
		send_command(ev_repaint);
		e.kill();
		return;
	}
	
	if (e.type == Event::keyboard)
		switch (e.data) {
		case '\n':
			e.kill();
			if (!run())
				break;
		case key_f(9):
		case 27:
			e.type = Event::command;
			e.data = ev_menu_stop;
			break;
		}
}

void Menu::handle_event(Event &e)
{
	if (e.type == Event::keyboard)
		switch (e.data) {
		// these two shall be handled by upper level form:
		case key_left:
		case key_right:
			return;
		// ignore
		case '\t':
		case '\t' | key_meta:
			e.kill();
			break;
		}
	Form::handle_event(e);
}

void MenuItem::do_draw(int c1, int c2)
{
	go(pos_x(), pos_y());
	
	if (caption == "-") {
		int w = pos.w;
		while (w--)
			print(get_frame(2,0), c1);
		return;
	}
	
	v_alt_print(caption, c1, c2,
		    pos.w - (v_alt_strlen(caption) + strlen(value)) + 
		    v_alt_strlen(caption));
	set_color(c1);
	print("%s", value.cstr());
	
	go(pos_x(), pos_y());
	
	if (active)
		active->draw();		// can have Menu child
}

void Menu::draw()
{
	int w = 0, k, i = 0;
	MenuItem *p;
	int c1 = color("menu_normal"), s1 = color("menu_selected"),
	    c2 = color("menu_hl_normal"), s2 = color("menu_hl_selected");
	
	for (p = (MenuItem*)children; p; p = (MenuItem*)p->next, i++)
		if ((k = v_alt_strlen(p->caption) + strlen(p->value)) > w)
			w = k;
			
	w += 2;
	pos.w = w + 2;
	pos.h = i + 2;
	
	pos.draw_frame_at(pos_x(), pos_y(), c1, true);
	
	i = 1;
	
	for (p = (MenuItem*)children; p; p = (MenuItem*)p->next) {
		p->pos.set(1, i++, w, 1);
		if (p != active)
			p->do_draw(c1, c2);
	}
	((MenuItem*)active)->do_draw(s1,s2);
	stick_cursor();
}

void MenuBar::link(Menu *p)
{
	if (children) {
		Menu *x = (Menu *)children;
		while (x->next)  x = (Menu *)x->next;
		p->pos.x = x->pos.x + v_alt_strlen(x->caption) + 2;
	} else {
		p->pos.x = 0;
	}
	p->pos.y = 1;
	add_child(p);
}

void MenuBar::draw()
{
	Menu *p;
	int n;

	int cn, cl, sn, sl;

	cn = color("menubar_normal");
	cl = color("menubar_hl_normal");
	sn = color("menubar_selected");
	sl = color("menubar_hl_selected");
	
	go(pos_x(), pos_y());
	n = s_width;
	while (n--)
		print(' ', cn);
		
	go(pos_x(), pos_y());
	for (p = (Menu*)children; p; p = (Menu*) p->next)
		if (p == active && is_active())
			v_alt_print(" " + p->caption + " ", sn, sl);
		else		
			v_alt_print(" " + p->caption + " ", cn, cl);
	if (is_active())
		active->draw();
}

void MenuBar::handle_event(Event &e)
{
	Widget *a = active;
	
	Form::handle_event(e);
	
	// bgd shall be repainted
	if (a != active)
		send_command(ev_repaint);
}

bool SwitchItem::run()
{
	int i;
	i = values.index(value) + 1;
	if (i == 0) i++;
	if (i >= values.size()) i = 0;
	value = values[i];
	send_command(ev_repaint);
	return false;
}

SwitchItem::SwitchItem(const String & c, const String & v,
		       const String & v1, const String & v2,
		       const String & v3, const String & v4,
		       const String & v5, const String & v6)
	: MenuItem(c)
{
	value = v;
	if (v1 != "") {
		values.push(v1);
		if (v2 != "") {
			values.push(v2);
			if (v3 != "") {
				values.push(v3);
				if (v4 != "") {
					values.push(v4);
					if (v5 != "") {
						values.push(v5);
						if (v6 != "") {
							values.push(v6);
						}
					}
				}
			}
		}
	}
}

bool MenuItem::is_accel(Event &e)
{
	return e.type == Event::keyboard && _v_alt_checkkey(caption, e.data);
}

bool Menu::is_accel(Event &e)
{
	if (e.type != Event::keyboard || (e.data & key_meta) == 0)
		return false;
	return _v_alt_checkkey(caption, (e.data & ~key_meta));
}

bool MenuBar::is_accel(Event &e)
{
	for (Widget *p = children; p; p = p->next)
		if (p->is_accel(e)) {
			activate(p);
			return true;
		}
	return false;
}

void MenuItem::link(Menu *m)
{
	children = m; 
	m->parent = this; 
	value = "> "; 
	m->pos.x = 2;
}
