/* This file can be redistributed and used under GNU Library General Public
 * License. (c) 2000 Micha Moskal <malekith@topnet.pl>. 
 * $Id: window.h,v 1.15 2000/09/20 08:24:55 malekith Exp $ */

#ifndef _DONT_INCLUDE_DIRECTLY
#error "window.h is intended to be included from <v/v.h>"
#endif

class Window : public Form {
	int start_x, start_y;
	bool dragging;
public:
	String title;
	bool close_button;
	Window() 
		: start_x(0), start_y(0), dragging(false), 
		close_button(true) 
		{ name = "Window"; }
	void center();
	
	virtual void draw();
	virtual void handle_event(Event &e);
	
	void async_draw();
};

class Dialog : public Window {
public:
	enum {
		ev_ok = ev_user,
		ev_cancel,
		ev_yes,
		ev_no,
		ev_dialog_user
	};
	int modal_result;
	// returns one of ev_*
	virtual int execute();
	virtual void handle_event(Event &e);
	Dialog() : modal_result(0) { name = "Dialog"; }
};

class StatusLine : public Widget {
public:
	String right_side, down;
	
	virtual void draw();
	virtual bool can_get_focus() { return false; }
};

class RootWindow : public Form {
	void handle_key(int k);
	Block <Widget *> winstack;
public:
	virtual void handle_event(Event &e);
	virtual void draw();
	virtual void kill_child(Widget *);
	virtual void activate(Widget *);
	RootWindow() : menu(0), antimenu(0), status_line(new StatusLine)
		 { name = "RootWindow"; help = "Kick v1.2"; }
	
	Widget *menu, *antimenu;
	StatusLine *status_line;
};


extern RootWindow v_root_window;

inline void Window::async_draw()
{
	draw();
	if (v_root_window.active != this)
		v_root_window.active->draw();
}

class TextArea : public Widget {
public:
	virtual void draw();
	
	// new:
	int fcolor;
	enum Align {
		left,
		right,
		center,
	} align;
	
	String caption;
	virtual bool can_get_focus() 
		{ return false; }
	TextArea() : fcolor(color("text_area")), align(left) 
		{ name = "TextArea"; }
	int word_break(int width, String &result);
};

// ---------------------------------------------------------------------------
// Menus
class Menu : public Form {
public:
	virtual void draw();
	virtual void handle_event(Event &e);
	String caption;
	bool is_accel(Event &e);
	
	Menu(const String &c = "") : caption(c) { name = "Menu"; }
};

class MenuItem : public Widget {
public:
	String caption, value;
	int cmd;
	
	virtual bool run();
		// shall return 1 if menu needs to be closed
	void do_draw(int c1, int c2);
	virtual bool can_get_focus() 
		{ if (caption == "-") return false; return true; }
	virtual void handle_event(Event &e);
	void link(Menu *m);
	virtual bool is_accel(Event &e);
	
	MenuItem(const String &c = "", int cm = 0, const String &h = "") 
		: caption(c), cmd(cm) 
		{ name = "MenuItem"; help = h; }
};

class MenuBar : public Form {
public:
	void link(Menu *p);
	virtual void draw();
	// cannot be focused by focus_next(), only by ev_menu_start
	virtual bool can_get_focus() { return false; }
	virtual void handle_event(Event &e);
	virtual bool is_accel(Event &e);
	
	MenuBar() { name = "MenuBar"; }
};

class SwitchItem : public MenuItem {
public:
	Block<String> values;
	virtual bool run();
	SwitchItem(const String &c = "", const String &v = "", 
		   const String &v1 = "", const String &v2 = "",
		   const String &v3 = "", const String &v4 = "",
		   const String &v5 = "", const String &v6 = "");
};

// ------------------------------------------------------------------------
// ProgressBars
class ProgressBar : public Widget {
	int filled, unfilled, sv_filled, sv_unfilled;
	void count();
public:
	int advance, max;
	bool frame;
	
	virtual void draw();
	bool changed();
	virtual bool can_get_focus() { return false; }
	ProgressBar(int x = 0, int y = 0, int w = 0, 
		    int _pos = 0, int _max = 100) :
		sv_filled(-1), sv_unfilled(-1), advance(_pos), max(_max) 
	{
		pos.set(x, y, w, 1);
		frame = true;
	}
};

class Label;
class ProgressWin : public Window {
public:
	Label *info;
	ProgressBar *pb;
	
	ProgressWin(int max = 100, const String &t = "Please wait ...", 
		    const String &i = "");
	
	void set(int p, int max = 0);
	void inc() { set(pb->advance + 1); }
};

class Box : public Widget {
public:
	String title;
	
	virtual bool can_get_focus() { return false; }
	Box(int x, int y, int w, int h, const String &c) : title(c)
		{ pos.set(x,y,w,h); }
	virtual void draw();
};
